# =========================================================================== #
# Gemspec for Project Bioroebe.
# =========================================================================== #
require 'bioroebe/version/version.rb'
require 'roebe'

Gem::Specification.new { |s|

  s.name    = 'bioroebe'
  s.version = Bioroebe.version
  s.date    = Time.now.strftime('%Y-%m-%d')

  DESCRIPTION = <<-EOF

DO NOT YET DOWNLOAD! THIS WILL BECOME THE NEW bioroebe-0.10.x RELEASE
SERIES IN A FEW DAYS! RIGHT NOW IT IS UNSTABLE AND WILL CHANGE A LOT.

Welcome to the bioroebe-0.10.x release series! It is recommended to
use ruby 3.x for this series, as it is the most tested version of 
ruby in regards to this project.

The bioroebe-project is a software suite focusing on bio-related
life sciences in general - in particular molecular biology, molecular
genetics, bioinformatics, biotechnology, system biology, synthetic
biology and related topics close to the life sciences in general.

The primary objective for the bioroebe project is to provide a
"practical glue" between these different topics while also focusing
on "getting real work done", as a toolset-project, trying to
efficiently solve existing problems.

For additional information about this project, have a look at the
link called "documentation" on the rubygems webpage of this gem,
on the very bottom right side (The direct URL is: 
https://www.rubydoc.info/gems/bioroebe/ ).

  EOF

  s.summary     = DESCRIPTION
  s.description = DESCRIPTION

  s.authors     = ['Robert A. Heiler']
  s.email       = Roebe.email?
  s.files       = Dir['**/*']
  s.files      << Dir['./yaml/*yml']
  s.files      << 'README.md'
  s.files       = s.files.flatten
  s.homepage    = 'https://rubygems.org/gems/bioroebe'
  s.licenses    = 'LGPL-2.1'
  s.executables = Dir['bin/*'].map { |file| File.basename(file) } 

  # ========================================================================= #
  # Show this when a user installs this project.
  # ========================================================================= #
  s.post_install_message = <<-EOF

This project relates to the life sciences, in particular molecular biology
and bioinformatics. It is primarily a toolset project, meaning that the
code that can be found within this project, ought to be "useful" in daily
work setups and routines related to the field of life sciences.

The simplest way to use this project is to start the interactive shell,
called BioShell.

The executable file for the BioShell is simply called:

    bioshell

It resides in the bin/ subdirectory of this gem. It can also be started
from ruby code, such as through the following two lines of ruby code:

    require 'bioroebe'
    Bioroebe.start_shell

Ideally this should work just fine. If not, have a look at your specific
GEM_PATH to find out where the bioroebe-related files are kept.

"gem install bioroebe" should properly have handled all dependencies
for the bioroebe gem.

For more documentation, have a look at:

  #{Bioroebe::URL_TO_THE_DOCUMENTATION}

  EOF

  s.required_ruby_version     = '>= '+Roebe.third_most_stable_version_of_ruby
  s.required_rubygems_version = '>= '+Gem::VERSION
  s.rubygems_version          = '>= '+Gem::VERSION

  # ========================================================================= #
  # Specify the dependencies, alphabetically sorted, of this project:
  # ========================================================================= #
  s.add_dependency 'chemistry_paradise'
  s.add_dependency 'cyberweb' # ← Added this dependency in June 2021. Let's see.
  s.add_dependency 'colours'
  s.add_dependency 'convert_global_env'
  s.add_dependency 'directory_paradise'
  s.add_dependency 'opn'
}