This file here attempts to note down what should be possible and doable 
within the Bioroebe main sequence class.

It is closely modelled towards this file here from the bioruby project:

    https://raw.githubusercontent.com/bioruby/bioruby/master/doc/Tutorial.rd

There will not be a lot of explanation going on here in this file - the
whole purpose of this file is to simply show what is possible with the
Bioroebe sequence, in a succinct manner.

For a more extensive explanation, either consider looking at the online
documentation for the various methods/classes, or simply have a look at
the .md file (README.md) that is distributed with the bioroebe-gem
as such.

    require 'bioroebe'

    seq = Bioroebe::Sequence.new("ATGCATGCAAAA")
    seq.complement                 # => "TTTTGCATGCAT"
    seq.subseq(3,8)                # => "GCATGC"
    seq.composition                # => {"A"=>6, "T"=>2, "G"=>2, "C"=>2}
    seq.gc_percent                 # => 33
    seq.translate                  # => "MHAK"
    seq.translate(2)               # => "CMQ" # This translates from frame 2
    seq.translate(:frame_two)      # => "CMQ" # Alias to the above.
    seq.translate(:from_frame_two) # => "CMQ" # Alias to the above.
    seq.translate.names            # => ["methionine", "histidine", "alanine", "lysine"]
    seq.translate.codes            # => ["Met", "His", "Ala", "Lys"]
    seq.translate(1,11)            # codon table 11
    seq.translate(2)               # translate from frame 2 ==> "CMQ"
    seq.translate(1,11)            # codon table 11 ==> "MHAK"
    seq.translate.composition      # => {"K"=>1, "A"=>1, "M"=>1, "H"=>1}
    seq.translate.molecular_weight # => 485.605
    seq.complement.translate       # => "FCMH"
